<?php

namespace Drupal\graphql_core_schema\Plugin\GraphQL\DataProducer;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;

/**
 * Returns the entity in a given language with optional fallback.
 *
 * @DataProducer(
 *   id = "entity_translation_fallback",
 *   name = @Translation("Entity translation fallback"),
 *   description = @Translation("Returns the translated entity with optional fallback."),
 *   produces = @ContextDefinition("entity",
 *     label = @Translation("Translated entity")
 *   ),
 *   consumes = {
 *     "entity" = @ContextDefinition("entity",
 *       label = @Translation("Entity")
 *     ),
 *     "language" = @ContextDefinition("string",
 *       label = @Translation("Language"),
 *       required = FALSE
 *     ),
 *     "fallback" = @ContextDefinition("boolean",
 *       label = @Translation("Return same entity as fallback"),
 *       required = FALSE
 *     )
 *   }
 * )
 */
class EntityTranslationFallback extends DataProducerPluginBase {

  /**
   * Resolver.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param string $language
   * @param bool|null $access
   * @param \Drupal\Core\Session\AccountInterface|null $accessUser
   * @param string|null $accessOperation
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   */
  public function resolve(EntityInterface $entity, ?string $language, ?bool $fallback) {
    if (!$language) {
      return $entity;
    }
    if ($entity instanceof TranslatableInterface && $entity->isTranslatable() && $entity->hasTranslation($language)) {
      $entity = $entity->getTranslation($language);
      $entity->addCacheContexts(["static:language:{$language}"]);
      /** @var \Drupal\Core\Access\AccessResultInterface $accessResult */
      $accessResult = $entity->access('view', NULL, TRUE);
      if (!$accessResult->isAllowed()) {
        return NULL;
      }
      return $entity;
    }

    // Fallback to returning same entity in current language.
    if ($fallback) {
      return $entity;
    }

    return NULL;
  }

}
